@@ -118,7 +118,7 @@ num_udev_devices(void)
 
     enumerate = udev_enumerate_new(udev);
     g_assert(enumerate);
-    
+
     // NB: using libudev here instead of GUdev so that we can check the return
     // value of udev_enumerate_scan_devices().
     // https://github.com/martinpitt/umockdev/issues/144
@@ -1031,6 +1031,8 @@ t_testbed_libc(UMockdevTestbedFixture * fixture, UNUSED_DATA)
     char *path;
     char pathbuf[PATH_MAX];
     int dirfd, fd;
+    struct stat st;
+    uid_t uid = getuid();
 
     /* start with adding one device */
     success = umockdev_testbed_add_from_string(fixture->testbed,
@@ -1111,6 +1113,56 @@ t_testbed_libc(UMockdevTestbedFixture * fixture, UNUSED_DATA)
     g_assert_cmpint(openat(AT_FDCWD, "sys/devices", O_RDONLY), <, 0);
     g_assert_cmpint(errno, ==, ENOENT);
     g_assert_cmpint(openat64(AT_FDCWD, "sys/devices", O_RDONLY), <, 0);
+
+    /* stat */
+    g_assert_cmpint(stat ("/sys/bus/pci/devices", &st), ==, 0);
+    g_assert_cmpint(st.st_nlink, ==, 2);
+    g_assert_cmpuint(st.st_uid, ==, uid);
+    g_assert(S_ISDIR(st.st_mode));
+
+    g_assert_cmpint(lstat ("/sys/bus/pci/devices/dev1", &st), ==, 0);
+    g_assert_cmpint(st.st_nlink, ==, 1);
+    g_assert_cmpuint(st.st_uid, ==, uid);
+    g_assert(S_ISLNK(st.st_mode));
+
+    g_assert_cmpint(stat ("/sys/bus/pci/devices/dev1", &st), ==, 0);
+    g_assert_cmpint(st.st_nlink, ==, 2);
+    g_assert_cmpuint(st.st_uid, ==, uid);
+    g_assert(S_ISDIR(st.st_mode));
+
+    g_assert_cmpint(fstatat (AT_FDCWD, "/sys/bus/pci/devices", &st, 0), ==, 0);
+    g_assert_cmpint(st.st_nlink, ==, 2);
+    g_assert_cmpuint(st.st_uid, ==, uid);
+    g_assert(S_ISDIR(st.st_mode));
+
+    g_assert_cmpint(fstatat (AT_FDCWD, "/sys/bus/pci/devices/dev1", &st, AT_SYMLINK_NOFOLLOW), ==, 0);
+    g_assert_cmpint(st.st_nlink, ==, 1);
+    g_assert_cmpuint(st.st_uid, ==, uid);
+    g_assert(S_ISLNK(st.st_mode));
+
+    g_assert_cmpint(fstatat (AT_FDCWD, "/sys/bus/pci/devices/dev1", &st, 0), ==, 0);
+    g_assert_cmpint(st.st_nlink, ==, 2);
+    g_assert_cmpuint(st.st_uid, ==, uid);
+    g_assert(S_ISDIR(st.st_mode));
+
+#ifdef __GLIBC__
+    /* statx */
+    struct statx stx;
+    g_assert_cmpint(statx (AT_FDCWD, "/sys/bus/pci/devices", 0, STATX_TYPE|STATX_NLINK|STATX_UID, &stx), ==, 0);
+    g_assert_cmpint(stx.stx_nlink, ==, 2);
+    g_assert_cmpuint(stx.stx_uid, ==, uid);
+    g_assert(S_ISDIR(stx.stx_mode));
+
+    g_assert_cmpint(statx (AT_FDCWD, "/sys/bus/pci/devices/dev1", AT_SYMLINK_NOFOLLOW, STATX_TYPE|STATX_NLINK|STATX_UID, &stx), ==, 0);
+    g_assert_cmpint(stx.stx_nlink, ==, 1);
+    g_assert_cmpuint(stx.stx_uid, ==, uid);
+    g_assert(S_ISLNK(stx.stx_mode));
+
+    g_assert_cmpint(statx (AT_FDCWD, "/sys/bus/pci/devices/dev1", 0, STATX_TYPE|STATX_NLINK|STATX_UID, &stx), ==, 0);
+    g_assert_cmpint(stx.stx_nlink, ==, 2);
+    g_assert_cmpuint(stx.stx_uid, ==, uid);
+    g_assert(S_ISDIR(stx.stx_mode));
+#endif
 }
 
 static void
@@ -1172,6 +1224,7 @@ t_testbed_dev_access(UMockdevTestbedFixture * fixture, UNUSED_DATA)
     int fd;
     FILE* f;
     char buf[100];
+    uid_t uid = getuid();
 
     /* no mocked devices */
     g_assert_cmpint(g_open("/dev/wishyouwerehere", O_RDONLY, 0), ==, -1);
@@ -1198,8 +1251,23 @@ t_testbed_dev_access(UMockdevTestbedFixture * fixture, UNUSED_DATA)
     g_assert_cmpint(g_open("/dev/wishyouwerehere", O_RDONLY, 0), ==, -1);
     g_assert_cmpint(errno, ==, ENOENT);
     g_assert_cmpint(g_stat("/dev/zero", &st), ==, 0);
+    g_assert_cmpuint(st.st_uid, ==, uid);
     g_assert(S_ISCHR(st.st_mode));
     g_assert_cmpint(st.st_rdev, ==, 0);	/* we did not set anything */
+
+    memset(&st, 42, sizeof st);
+    g_assert_cmpint(fstatat(AT_FDCWD, "/dev/zero", &st, 0), ==, 0);
+    g_assert_cmpuint(st.st_uid, ==, uid);
+    g_assert(S_ISCHR(st.st_mode));
+    g_assert_cmpint(st.st_rdev, ==, 0);	/* we did not set anything */
+
+#ifdef __GLIBC__
+    struct statx stx;
+    g_assert_cmpint(statx (AT_FDCWD, "/dev/zero", 0, STATX_TYPE|STATX_UID, &stx), ==, 0);
+    g_assert_cmpuint(stx.stx_uid, ==, uid);
+    g_assert(S_ISCHR(stx.stx_mode));
+#endif
+
     fd = g_open("/dev/zero", O_RDONLY, 0);
     g_assert_cmpint(fd, >, 0);
     g_assert(!isatty(fd));
@@ -1352,6 +1420,18 @@ t_testbed_add_from_string_dev_block(UMockdevTestbedFixture * fixture, UNUSED_DAT
     g_assert_cmpint(g_stat("/dev/empty", &st), ==, 0);
     g_assert(S_ISBLK(st.st_mode));
 
+    memset(&st, 42, sizeof st);
+    g_assert_cmpint(fstatat(AT_FDCWD, "/dev/empty", &st, 0), ==, 0);
+    g_assert_cmpuint(st.st_uid, ==, getuid());
+    g_assert(S_ISBLK(st.st_mode));
+
+#ifdef __GLIBC__
+    struct statx stx;
+    g_assert_cmpint(statx (AT_FDCWD, "/dev/empty", 0, STATX_TYPE|STATX_UID, &stx), ==, 0);
+    g_assert_cmpuint(stx.stx_uid, ==, getuid());
+    g_assert(S_ISBLK(stx.stx_mode));
+#endif
+
     /* N: with value should set that contents */
     g_assert(umockdev_testbed_add_from_string(fixture->testbed,
 					      "P: /devices/block/filled\n"
@@ -1366,6 +1446,12 @@ t_testbed_add_from_string_dev_block(UMockdevTestbedFixture * fixture, UNUSED_DAT
     g_free(contents);
     g_assert_cmpint(g_stat("/dev/sdf", &st), ==, 0);
     g_assert(S_ISBLK(st.st_mode));
+
+#ifdef __GLIBC__
+    g_assert_cmpint(statx (AT_FDCWD, "/dev/sdf", 0, STATX_TYPE|STATX_UID, &stx), ==, 0);
+    g_assert_cmpuint(stx.stx_uid, ==, getuid());
+    g_assert(S_ISBLK(stx.stx_mode));
+#endif
 }
 
 static void
